## copy the labels to a single array
from whacc import utils
import numpy as np
import h5py
from natsort import os_sorted
import copy

label_files = os_sorted(utils.get_files("/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/", "*_labels.npy"))
label_files = utils.lister_it(label_files, keep_strings=None, remove_string='curated')

all_labels = []
for k in label_files:
    all_labels.append(np.load(k)[0])

h5_file = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000.h5"
with h5py.File(h5_file, 'r') as hf:
    frame_nums = copy.deepcopy(hf['trial_nums_and_frame_nums'][1, :])

new_labels = np.zeros(int(np.sum(frame_nums))).astype(int)
for ii, (i1, i2) in enumerate(utils.loop_segments(frame_nums)):
    print(ii)
    for k in all_labels[ii]:
        new_labels[i1:i2][int(k-1)] = 1


h5_file = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000.h5"
with h5py.File(h5_file, 'r+') as hf:
    hf['labels'][:] = new_labels

##################
# plot random touch onsets
import matplotlib.pyplot as plt
from whacc import image_tools

all_onsets = np.where(np.diff(new_labels) == 1)[0]+1
add2 = 2
h5_file = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000.h5"
num_plots = 10
with h5py.File(h5_file, 'r') as hf:
    for k in np.random.choice(all_onsets, num_plots):
        plt.figure()
        plt.imshow(image_tools.img_unstacker(hf['images'][k-add2 : k+add2+1], num_frames_wide=1+add2*2))
        print(k-add2,  k+add2+1, k)


#######
# make a new reduced sized only near touches file with only one image deep
from whacc import utils
from whacc import image_tools
import copy
import h5py
f = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000.h5'
f2 = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single.h5'

h5c = image_tools.h5_iterative_creator(f2, overwrite_if_file_exists=True, color_channel=False)

with h5py.File(f, 'r') as h:
    print(len(h['labels'][:]))
    h_cont = copy.deepcopy(h['labels'][:])
utils.create_master_dataset(h5c, [f], [h_cont], borders=80, max_pack_val=100)






# everything you need to make the below plot is here
# plot the histogram for the comparing the error types
a = []
for k in list(itertools.permutations(range(4), 2)):
  a.append(analysis.error_analysis(h_cont_1[k[1], :], h_cont_1[k[0], :], frame_num_array=frame_num_array.astype(int)))

def get_error_str_list(a):
  all_errors = []
  for k in a.error_neg:
    all_errors.append(a.type_list[k])
  for k in a.error_pos:
    all_errors.append(a.type_list[k])
  return all_errors

a_dict = dict()
a_dict['Count'] = []
a_dict['Error type'] = []
a_dict['pairs'] = []
# a_dict['count'] = []
for i, k in enumerate(list(itertools.permutations(range(4), 2))):
  if 3 in k:
    out = get_error_str_list(a[i])
    out = out+['append', 'deduct', 'ghost', 'join', 'miss', 'split']
    keys, counts = np.unique(out, return_counts=True)
    counts = counts-1
    a_dict['Count'] += list(counts)
    a_dict['Error type'] += list(keys)
    a_dict['pairs'] += list(np.tile('pairs '+str(k), len(keys)))

for i, k in enumerate(list(itertools.permutations(range(4), 2))):
  if 3 not in k:
    out = get_error_str_list(a[i])
    out = out+['append', 'deduct', 'ghost', 'join', 'miss', 'split']
    keys, counts = np.unique(out, return_counts=True)
    counts = counts-1
    a_dict['Count'] += list(counts)
    a_dict['Error type'] += list(keys)
    a_dict['pairs'] += list(np.tile('pairs '+str(k), len(keys)))


import pandas as pd
import datetime
import seaborn as sns

df = pd.DataFrame.from_dict(a_dict)
plt.figure(figsize=(15, 10))
# plt.ylim([0, 300])
# plt.yscale("log")
sns.barplot(x="Error type", hue="pairs", y="Count", data=df)




# # # import h5py
# # # import matplotlib.pyplot as plt
# # # from whacc import image_tools
# # # import os
# # # from whacc import utils
# # #
# # # for h5_file in utils.get_h5s('/Users/phil/Dropbox/Autocurator/data/samsons_subsets/OLD_2'):
# # #     with h5py.File(h5_file, 'r') as h:
# # #         plt.figure(figsize=[20, 10])
# # #         plt.imshow(image_tools.img_unstacker(h['images'][:],40))
# # #         plt.title(h5_file.split('/')[-1])
# # #         plt.savefig(h5_file[:-3] + "_fig_plot.png")
# # #         plt.close()
# # #
# # #
# # # for k in utils.get_h5s('/Users/phil/Dropbox/Autocurator/data/samsons_subsets/OLD_2'):
# # #     with h5py.File(k, 'r') as h:
# # #         k2 = k[:-3]+'_TMP.h5'
# # #         with h5py.File(k2, 'w') as h2:
# # #             h2.create_dataset('all_inds', data=h['all_inds'][50:])
# # #             h2.create_dataset('images', data=h['images'][50:])
# # #             h2.create_dataset('in_range', data=h['in_range'][50:])
# # #             h2.create_dataset('labels', data=h['labels'][50:])
# # #             h2.create_dataset('retrain_H5_info', data=h['retrain_H5_info'][:])
# # #     os.remove(k)
# # #     os.rename(k2, k)
# # #
# # #
# # # import h5py
# # # import matplotlib.pyplot as plt
# # # from whacc import image_tools
# # #
# # # h5_file= '/Users/phil/Downloads/AH1159X18012021xS404_subset (1).h5'
# # # with h5py.File(h5_file, 'r') as h:
# # #     plt.figure(figsize=[20, 10])
# # #     plt.imshow(image_tools.img_unstacker(h['images'][:],10))
# # #     plt.title(h5_file.split('/')[-1])
# # #     print(len(h['labels'][:]))
# # #
# # #
# # #
# # #
# #
# # import numpy as np
# # import whacc
# # from whacc import analysis
# # import matplotlib.pyplot as plt
# #
# #
# # a = analysis.pole_plot(
# #     '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/master_train_test1.h5',
# #     pred_val = [0,0,0,0,0,0,0,.2,.4,.5,.6,.7,.8,.8,.6,.4,.2,.1,0,0,0,0],
# #     true_val = [0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0],
# #     len_plot = 10)
# #
# # a.plot_it()
# #
# #
# f1 = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/delete_after_oct_2021/AH0000x000000.h5"
# import h5py
#
# with h5py.File(f1, 'r') as h:
#     k = 'file_name_nums'
#     print(k, '   ', h[k].shape)
#     k = 'full_file_names'
#     print(k, '   ', (h[k].shape))
#     k = 'images'
#     print(k, '   ', h[k].shape)
#     k = 'in_range'
#     print(k, '   ', h[k].shape)
#     k = 'labels'
#     print(k, '   ', h[k].shape)
#     k = 'locations_x_y'
#     print(k, '   ', h[k].shape)
#     k = 'max_val_stack'
#     print(k, '   ', h[k].shape)
#     k = 'multiplier'
#     print(k, '   ', h[k].shape)
#     k = 'trial_nums_and_frame_nums'
#     print(k, '   ', h[k].shape)
#
# with h5py.File(save_directory + file_name, 'r+') as hf:  # with -> auto close in case of failure
#     hf.create_dataset("asdfasdf", data=loc_stack_all2)
#     for i, k in enumerate(loc_stack_all):
#         print(i)
#         hf.create_dataset("aaa"+str(i), data=k)
#
# loc_stack_all2 = loc_stack_all.copy()
#
#
#
#
# for i, k in enumerate(loc_stack_all):
#     if not k.shape==(4000, 2):
#         print(i)
#     # for kk in k.flatten():
#     #     print(kk.shape)
#     #         # if not kk.dtype=='int64':
#     #         #     print('booooo')
#     #
#     #


# import h5py
# import numpy as np
# import matplotlib.pyplot as plt
# cnt = 0
# f = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000.h5"
# with h5py.File(f, 'r') as h:
#     print(len(h['labels']))
#     for k in range(0, 1163998, 20000):
#         plt.figure()
#         plt.imshow(h['images'][k])
#         cnt+=1
#         # if cnt>=20 :
#         #     asdfasdf

from whacc import utils
from whacc import image_tools
import copy
import h5py
f = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000.h5'
f2 = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single_frame.h5'

h5c = image_tools.h5_iterative_creator(f2,
                 overwrite_if_file_exists=False,
                 max_img_height=61,
                 max_img_width=61,
                 close_and_open_on_each_iteration=True,
                 color_channel=False)
with h5py.File(f, 'r') as h:
    print(len(h['labels'][:]))
    h_cont = copy.deepcopy(h['labels'][:])
utils.create_master_dataset(h5c, [f], [h_cont], borders=80, max_pack_val=100)


####


f = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single_frame.h5"
with h5py.File(f, 'r') as h:
    utils.print_list_with_inds(h.keys())
    tmp1 = copy.deepcopy(h['frame_nums'][:])



from whacc import utils
from whacc import image_tools
import numpy as np
import h5py
import matplotlib.pyplot as plt
import shutil
from tqdm import tqdm
import copy

#  make a lagged H5 file where frame 1 is stacked 3 deep 1 is (rand, rand, 1) 2 is (rand, 1, 2) 3 is (1, 2, 3)

f = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single_frame.h5"
f2 = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single_frame_lag_3_deep_real_at_end.h5"

h5c = image_tools.h5_iterative_creator(f2, overwrite_if_file_exists=True)

with h5py.File(f, 'r+') as h:
    x = h['frame_nums'][:]
    for ii, (k1, k2) in enumerate(tqdm(utils.loop_segments(x), total=len(x))):
        new_imgs = image_tools.stack_imgs_lag(h['images'][k1:k2])
        h5c.add_to_h5(new_imgs, np.ones(new_imgs.shape[0]) * -1)

# copy over the other info from the OG h5 file
utils.copy_h5_key_to_another_h5(f, f2, 'labels')
utils.copy_h5_key_to_another_h5(f, f2, 'frame_nums')

# make a copy of the file and rename it
f3 = f2[:-3] + '_DIFF.h5'
shutil.copy(f2, f3)
# change color channel 0 and 1 to diff images from color channel 3 so color channels 0, 1, and 2 are 0-2, 1-2, and 2
with h5py.File(f3, 'r+') as h:
    for i in tqdm(range(h['images'].shape[0])):
        k = copy.deepcopy(h['images'][i])
        for img_i in range(2):
            k = k.astype(float)
            a = k[:, :, img_i] - k[:, :, -1]
            a = ((a + 255) / 2).astype(np.uint8)
            h['images'][i, :, :, img_i] = a

# plot and example of the new H5 files so we know what we are dealing with
dis_file = f3
with h5py.File(dis_file, 'r') as h:
    ind1 = 80
    print(h['images'][ind1].shape)
    plt.imshow(h['images'][ind1])
    plt.figure()
    plt.plot(h['labels'][:4000])

    plt.figure()
    plt.imshow(h['images'][ind1][:, :, 0])
    plt.figure()
    plt.imshow(h['images'][ind1][:, :, 1])
    plt.figure()
    plt.imshow(h['images'][ind1][:, :, 2])

    utils.print_list_with_inds(h.keys())

"""

"""
from whacc import analysis
a = analysis.pole_plot()

def four_class_labels_from_binary(x):
    a = np.asarray(x)
    b = np.asarray([0]+list(np.diff(a)))
    c = a+b
    c[c==-1] = 3
    return c
x = [0,0,0,0,0,1,1,1,1,1,0,0,0,0,0]
four_class_labels_from_binary(x)


h5tmp = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single_frame_lag_3_deep_real_at_end.h5"
a = analysis.pole_plot(h5tmp, true_val=image_tools.get_h5_key_and_concatenate([tmph5], 'labels'))
a.current_frame = 75
a.plot_it()
plt.ylim([-.5, 3.5])
plt.show()
# 000002111130000



